using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore.Migrations;
using Microsoft.EntityFrameworkCore.Migrations.Operations;
using Microsoft.EntityFrameworkCore.Migrations.Operations.Builders;
using Oqtane.Databases.Interfaces;
using Oqtane.Migrations;
using Oqtane.Migrations.EntityBuilders;

namespace SZUAbsolventenverein.Module.EventRegistration.Migrations.EntityBuilders
{
    public class EventResponseEntityBuilder : AuditableBaseEntityBuilder<EventResponseEntityBuilder>
    {
        private const string _entityTableName = "SZUAbsolventenvereinEventResponse";
        private readonly PrimaryKey<EventResponseEntityBuilder> _primaryKey = new("PK_SZUAbsolventenvereinEventResponse", x => x.EventResponseId);
        private readonly ForeignKey<EventResponseEntityBuilder> _moduleForeignKey = new("FK_SZUAbsolventenvereinEventResponse_Module", x => x.ModuleId, "Module", "ModuleId", ReferentialAction.Cascade);
        private readonly ForeignKey<EventResponseEntityBuilder> _eventForeignKey = new("FK_SZUAbsolventenvereinEventResponse_Event", x => x.EventRegistrationId, "SZUAbsolventenvereinEvent", "EventId", ReferentialAction.Cascade);
        private readonly ForeignKey<EventResponseEntityBuilder> _ownerForeignKey = new("FK_SZUAbsolventenvereinEventResponse_User_Owner", x => x.OwnerId, "User", "UserId", ReferentialAction.Cascade);

        public EventResponseEntityBuilder(MigrationBuilder migrationBuilder, IDatabase database) : base(migrationBuilder, database)
        {
            EntityTableName = _entityTableName;
            PrimaryKey = _primaryKey;
            ForeignKeys.Add(_moduleForeignKey);
            ForeignKeys.Add(_eventForeignKey);
            ForeignKeys.Add(_ownerForeignKey);
        }

        protected override EventResponseEntityBuilder BuildTable(ColumnsBuilder table)
        {
            EventResponseId = AddAutoIncrementColumn(table, "EventResponseId");
            ResponseType = AddBooleanColumn(table, "ResponseType");
            OwnerId = AddIntegerColumn(table, "OwnerId");
            EventRegistrationId = AddIntegerColumn(table,"EventRegistrationId");
            ModuleId = AddIntegerColumn(table,"ModuleId");
            AddAuditableColumns(table);
            return this;
        }

        public OperationBuilder<AddColumnOperation> EventResponseId { get; set; }
        public OperationBuilder<AddColumnOperation> ResponseType { get; set; }
        public OperationBuilder<AddColumnOperation> OwnerId { get; set; }
        public OperationBuilder<AddColumnOperation> EventRegistrationId { get; set; }
        public OperationBuilder<AddColumnOperation> ModuleId { get; set; }
    }
}
